__all__ = (
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__ru__",
    "__author__en__",
    "__email__",
    "__license__",
    "__copyright__",
    "__maintainer__ru__",
    "__maintainer__en__",
    "__maintainer_email__",
)

__title__ = "OCEAN-AI"
__summary__ = "OCEAN-AI"
__uri__ = "https://github.com/DmitryRyumin/oceanai"

__version__ = "1.0"
__release__ = __version__ + ".0a39"

__author__ru__ = "Рюмина Елена, Рюмин Дмитрий, Карпов Алексей"
__author__en__ = "Elena Ryumina, Dmitry Ryumin, Alexey Karpov"
__email__ = "ryumina_ev@mail.ru, dl_03.03.1991@mail.ru, karpov@iias.spb.su"

__maintainer__ru__ = "Рюмина Елена, Рюмин Дмитрий"
__maintainer__en__ = "Elena Ryumina, Dmitry Ryumin"
__maintainer_email__ = "ryumina_ev@mail.ru, dl_03.03.1991@mail.ru"

__license__ = "BSD License"
__copyright__ = "2022-24. Speech and Multimodal Interfaces Laboratory Team"
